package com.zavoo.svg
{
	public class SvgMath
	{
			
		static public function cubicBezierToQuadratic(P0:Object, P1:Object, P2:Object, P3:Object):Array
		{
			/* A portion of code from Bezier_lib.as by Timothee Groleau */
			// calculates the useful base points
			var PA:Object = getPointOnSegment(P0, P1, 3/4);
			var PB:Object = getPointOnSegment(P3, P2, 3/4);
			
			// get 1/16 of the [P3, P0] segment
			var dx:Number = (P3.x - P0.x)/16;
			var dy:Number = (P3.y - P0.y)/16;
			
			// calculates control point 1
			var Pc_1:Object = getPointOnSegment(P0, P1, 3/8);
			
			// calculates control point 2
			var Pc_2:Object = getPointOnSegment(PA, PB, 3/8);
			Pc_2.x -= dx;
			Pc_2.y -= dy;
			
			// calculates control point 3
			var Pc_3:Object = getPointOnSegment(PB, PA, 3/8);
			Pc_3.x += dx;
			Pc_3.y += dy;
			
			// calculates control point 4
			var Pc_4:Object = getPointOnSegment(P3, P2, 3/8);
			
			// calculates the 3 anchor points
			var Pa_1:Object = getMiddle(Pc_1, Pc_2);
			var Pa_2:Object = getMiddle(PA, PB);
			var Pa_3:Object = getMiddle(Pc_3, Pc_4);
			
			// draw the four quadratic subsegments
			return ([{cx:Pc_1.x, cy:Pc_1.y, ax:Pa_1.x, ay:Pa_1.y},
					{cx:Pc_2.x, cy:Pc_2.y, ax:Pa_2.x, ay:Pa_2.y},
					{cx:Pc_3.x, cy:Pc_3.y, ax:Pa_3.x, ay:Pa_3.y},
					{cx:Pc_4.x, cy:Pc_4.y, ax:P3.x, ay:P3.y}]);
						
		}
			
		static public function getMiddle(P0:Object, P1:Object):Object
		{
			/* A portion of code from Bezier_lib.as by Timothee Groleau */
			return {x: ((P0.x + P1.x) / 2), y: ((P0.y + P1.y) / 2)};
		}	
		
		static public function getPointOnSegment(P0:Object, P1:Object, ratio:Number):Object 
		{
			/* A portion of code from Bezier_lib.as by Timothee Groleau */
			return {x: (P0.x + ((P1.x - P0.x) * ratio)), y: (P0.y + ((P1.y - P0.y) * ratio))};
		}
		
		public function ellipticalArcToCurves(xRadius:Number, yRadius:Number, arc:Number, startAngle:Number, x:Number, y:Number, curves:Array):Object
		{
			/* Portions of code taken from drawArc.as by Ric Ewing */
			if (Math.abs(arc)>360) {
				arc = 360;
			}
			// Flash uses 8 segments per circle, to match that, we draw in a maximum
			// of 45 degree segments. First we calculate how many segments are needed
			// for our arc. 
			var segs:Number = Math.ceil(Math.abs(arc)/45);
			// Now calculate the sweep of each segment
			var segAngle:Number = arc/segs;
			// The math requires radians rather than degrees. To convert from degrees
			// use the formula (degrees/180)*Math.PI to get radians. 
			var theta:Number = -(segAngle/180)*Math.PI;
			// convert angle startAngle to radians
			var angle:Number = -(startAngle/180)*Math.PI;
			// find our starting points (ax,ay) relative to the secified x,y
			var ax:Number = x-Math.cos(angle)*xRadius;
			var ay:Number = y-Math.sin(angle)*yRadius;
			// if our arc is larger than 45 degrees, draw as 45 degree segments
			// so that we match Flash's native circle routines.
			if (segs>0) {
				// Loop for drawing arc segments
				for (var i:int = 0; i<segs; i++) {
					// increment our angle
					angle += theta;
					// find the angle halfway between the last angle and the new
					var angleMid:Number = angle-(theta/2);
					// calculate our end point
					var bx:Number = ax+Math.cos(angle)*xRadius;
					var by:Number = ay+Math.sin(angle)*yRadius;
					// calculate our control point
					var cx:Number = ax+Math.cos(angleMid)*(xRadius/Math.cos(theta/2));
					var cy:Number = ay+Math.sin(angleMid)*(yRadius/Math.cos(theta/2));
					// draw the arc segment
					curves.push({cx:cx, cy:cy, ax:bx, ay:by});
				}
			}
			// In the native draw methods the user must specify the end point
			// which means that they always know where they are ending at, but
			// here the endpoint is unknown unless the user calculates it on their 
			// own. Lets be nice and let save them the hassle by passing it back. 
			return {x:bx, y:by};
		}		
	}
}