package rxb
{
	import flash.display.InteractiveObject;
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	import reflex.behaviors.Behavior;
	import reflex.data.IPosition;
	import reflex.data.IScroll;
	import reflex.data.Position;
	import reflex.events.ButtonEvent;
	import reflex.measurement.resolveHeight;
	
	//fromr reflex, just changed to work directly with values of : min, max, value and pageSize
	public class SlideBehavior extends Behavior// extends StepBehavior
	{
		
		[Bindable]
		[Binding(target="target.skin.track")]
		public var track:InteractiveObject;
		
		[Bindable]
		[Binding(target="target.skin.thumb")]
		public var thumb:InteractiveObject;
		
		[Bindable]
		[Binding(target="target.horizontal")]
		public var horizontal:Boolean = false;
		
		[Bindable]
		[Binding(target="target.min")]
		public var min:Number;
		[Bindable]
		[Binding(target="target.max")]
		public var max:Number;
		[Bindable]
		[Binding(target="target.value")]
		public var value:Number;
		[Bindable]
		[Binding(target="target.pageSize")]
		public var pageSize:Number;
		
		
		private var _percent:Number = 0;
		private var dragPercent:Number;
		private var dragPoint:Number;
		private var dragging:Boolean;
		private var forwardPress:Boolean;
		
		public function SlideBehavior(target:InteractiveObject = null)
		{
			super(target);
		}
		
		[Bindable(event="percentChange")]
		public function get percent():Number
		{
			return _percent;
		}
		
		override public function set target(value:IEventDispatcher):void
		{
			super.target = value;
			
			if (target == null) {
				return;
			}
			
			//track = getSkinPart("track");
			//thumb = getSkinPart("thumb");
			if(track) { ButtonEvent.initialize(track); }
			if(thumb) { ButtonEvent.initialize(thumb); }
			
			if (track && track.width > track.height) {
				horizontal = true;
			}
			if(track && thumb) {
				updatePosition();
			}
		}
		
		private var _snapThumb:Boolean = false;
		
		[Bindable(event="snapThumbChange")]
		public function get snapThumb():Boolean { return _snapThumb; }
		public function set snapThumb(value:Boolean):void {
			_snapThumb = value;
			dispatchEvent(new Event("snapThumbChange"));
		}
		
		[PropertyListener(target="position.percent")]
		public function onPosition(percent:Number):void
		{
			if (thumb == null || track == null) {
				return;
			}
			
			if (!dragging) {
				_percent = (value-min)/(max-min);
				updatePosition();
				dispatchEvent(new Event("percentChange"));
			}
		}
		
		[EventListener(type="press", target="track")]
		public function onTrackPress(event:ButtonEvent):void
		{
			var percent:Number = (value-min)/(max-min);
			var size:Number = horizontal ? track.width - thumb.width : track.height - thumb.height;
			var mousePoint:Number = horizontal ? track.parent.mouseX - track.x : track.parent.mouseY - track.y;
			
			if (snapThumb) {
				thumb.dispatchEvent(new MouseEvent(MouseEvent.MOUSE_DOWN, true, false, thumb.mouseX, thumb.mouseY));
				
				_percent = (mousePoint - thumb.width/2) / size;
				_percent = _percent <= 0 ? 0 : _percent >= 1 ? 1 : _percent;
				//position.percent = _percent;
				value = (max-min)*_percent + min;
				updatePosition();
				
				dragPoint = horizontal ? thumb.x + thumb.width/2 : thumb.y + thumb.height/2;
				dragPercent = _percent;
				
				dispatchEvent(new Event("percentChange"));
			} else {
				forwardPress = mousePoint > ((horizontal ? thumb.width/2 : thumb.height/2) + size*percent);
				
				
				if (forwardPress) {
					//control.skipForward();
					value += pageSize;
				} else {
					//control.skipBackward();
					value -= pageSize;
				}
				
			}
			event.updateAfterEvent();
		}
		
		[EventListener(type="hold", target="track")]
		public function onTrackHold(event:ButtonEvent):void
		{
			var percent:Number = (value-min)/(max-min);
			var size:Number = horizontal ? track.width - thumb.width : track.height - thumb.height;
			var mousePoint:Number = horizontal ? track.parent.mouseX - track.x : track.parent.mouseY - track.y;
			var forwardHold:Boolean = mousePoint > ((horizontal ? thumb.width/2 : thumb.height/2) + size*percent);
			
			if (forwardPress != forwardHold) {
				return;
			}
			
			if (forwardPress) {
				//control.skipForward();
				value += pageSize;
			} else {
				//control.skipBackward();
				value -= pageSize;
			}
			
			event.updateAfterEvent();
		}
		
		[EventListener(type="press", target="thumb")]
		public function onThumbPress(event:ButtonEvent):void
		{
			dragging = true;
			dragPoint = horizontal ? thumb.parent.mouseX : thumb.parent.mouseY;
			dragPercent = _percent;
		}
		
		[EventListener(type="drag", target="thumb")]
		public function onThumbDrag(event:ButtonEvent):void
		{
			var mousePoint:Number = horizontal ? thumb.parent.mouseX : thumb.parent.mouseY;
			var size:Number = horizontal ? track.width - thumb.width : track.height - thumb.height;
			var delta:Number = (mousePoint - dragPoint) / size;
			_percent = dragPercent + delta;
			_percent = _percent <= 0 ? 0 : (_percent >= 1 ? 1 : _percent);
			//position.percent = _percent;
			value = (max-min)*_percent + min;
			updatePosition();
			dispatchEvent(new Event("percentChange"));
			
			event.updateAfterEvent();
		}
		
		[EventListener(type="release", target="thumb")]
		[EventListener(type="releaseOutside", target="thumb")]
		public function onThumbRelease(event:ButtonEvent):void
		{
			dragging = false;
		}
		
		[PropertyListener(target="target.width")]
		[PropertyListener(target="target.height")]
		[PropertyListener(target="target.min")]
		[PropertyListener(target="target.max")]
		[PropertyListener(target="target.value")]
		[PropertyListener(target="target.pageSize")]
		public function onResize(size:Number):void
		{
			_percent = (value-min)/(max-min);
			updatePosition();
		}
		
		
		public function updatePosition():void
		{
			if(track && thumb) {
				var p:Point = new Point();
				
				if (horizontal) {
					p.x = (track.width - thumb.width) * _percent + track.x;
					p = thumb.parent.globalToLocal( track.parent.localToGlobal(p) );
					thumb.x = Math.round(p.x);
				} else {
					var trackHeight:Number = reflex.measurement.resolveHeight(track);
					var thumbHeight:Number = reflex.measurement.resolveHeight(thumb);
					p.y = (trackHeight - thumbHeight) * _percent + track.y;
					p = thumb.parent.globalToLocal( track.parent.localToGlobal(p) );
					thumb.y = Math.round(p.y);
				}
			}
		}
		
	}
}
